from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List

from bot_data.model.scheduled_event import ScheduledEvent
from bot_graphql.abc.filter_abc import FilterABC


class ScheduledEventFilter(FilterABC):
    def __init__(self, bot: DiscordBotServiceABC, services: ServiceProviderABC):
        FilterABC.__init__(self)
        self._bot = bot
        self._services = services

        self._id = None
        self._interval = None
        self._name = None
        self._description = None
        self._channel_id = None
        self._start_time = None
        self._end_time = None
        self._entity_type = None
        self._location = None
        self._server = None

    def from_dict(self, values: dict):
        self._get_attributes_from_dict(
            [
                ("id", int),
                ("interval", str),
                ("name", str),
                ("description", str),
                ("channel_id", str),
                ("start_time", str),
                ("end_time", str),
                ("entity_type", str),
                ("location", str),
            ],
            values,
        )

        if "server" in values:
            from bot_graphql.filter.server_filter import ServerFilter

            self._server: ServerFilter = self._services.get_service(ServerFilter)
            self._server.from_dict(values["server"])

    def filter(self, query: List[ScheduledEvent]) -> List[ScheduledEvent]:
        if self._id is not None:
            query = query.where(lambda x: x.id == self._id)

        query = self._filter_by_attributes(
            [
                lambda x: x.id == self._id,
                lambda x: x.interval == self._interval,
                lambda x: x.name == self._name,
                lambda x: x.description == self._description,
                lambda x: x.channel_id == self._channel_id,
                lambda x: x.start_time == self._start_time,
                lambda x: x.end_time == self._end_time,
                lambda x: x.entity_type == self._entity_type,
                lambda x: x.location == self._location,
            ],
            query,
        )

        if self._server is not None:
            servers = self._server.filter(query.select(lambda x: x.server)).select(lambda x: x.id)
            query = query.where(lambda x: x.server.id in servers)

        return query
